package com.qy.worksheetsystem.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.DecodedJWT;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;

import java.security.SecureRandom;
import java.util.Base64;
import java.util.Date;


/**
 * @author he
 * @date 2022/2/28
 * @description  JWT工具类
 */
public class JWTUtil {

    private static final String ALGORITHM = "AES";
    private static final int KEY_SIZE = 128;
    // 过期时间 24 小时
    public static final long EXPIRE_TIME = 60 * 24 * 60 * 1000;
    private static final int EXPIRE_TIME_S = 60 * 24 * 60;
    // 密钥
    private static final String SECRET = "hty_202401203";
    public static String encrypt(String data, String password) throws Exception {
        SecretKey key = generateKey(password);
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.ENCRYPT_MODE, key);
        byte[] encryptedData = cipher.doFinal(data.getBytes());
        return Base64.getEncoder().encodeToString(encryptedData);
    }

    public static String decrypt(String encryptedData,String password) throws Exception {
        SecretKey key = generateKey(password);
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        cipher.init(Cipher.DECRYPT_MODE, key);
        byte[] decryptedData = cipher.doFinal(Base64.getDecoder().decode(encryptedData));
        return new String(decryptedData);
    }
    private static SecretKey generateKey(String password) throws Exception {
        KeyGenerator keyGenerator = KeyGenerator.getInstance(ALGORITHM);
        keyGenerator.init(KEY_SIZE, new SecureRandom(password.getBytes()));
        return new SecretKeySpec(keyGenerator.generateKey().getEncoded(), ALGORITHM);
    }
    /**
     * 生成 token
     */
    public static String createToken(String number) {
        try {
            Date date = new Date(System.currentTimeMillis() + EXPIRE_TIME);
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            // 附带username信息
            String token = JWT.create()
                    .withClaim("number", number)
                    //到期时间
                    .withExpiresAt(date)
                    //创建一个新的JWT，并使用给定的算法进行标记
                    .sign(algorithm);
            HutoolCacheUtils.put(token,number,EXPIRE_TIME_S);
            return token;
        } catch (UnsupportedEncodingException e) {
            return null;
        }
    }
    public static String createToken(String number,String userid,long exp) {
        try {
            Date date = new Date(System.currentTimeMillis() + exp);
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            // 附带username信息
            String token = JWT.create()
                    .withClaim("number", number)
                    .withClaim("userid", userid)
                    //到期时间
                    .withExpiresAt(date)
                    //创建一个新的JWT，并使用给定的算法进行标记
                    .sign(algorithm);
            HutoolCacheUtils.put(token,number,(int)exp);
            return token;
        } catch (UnsupportedEncodingException e) {
            return null;
        }
    }
    public static String createToken(String number,String userid) {
        try {
            Date date = new Date(System.currentTimeMillis() + EXPIRE_TIME);
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            // 附带username信息
            String token = JWT.create()
                    .withClaim("number", number)
                    .withClaim("userid", userid)
                    //到期时间
                    .withExpiresAt(date)
                    //创建一个新的JWT，并使用给定的算法进行标记
                    .sign(algorithm);
            HutoolCacheUtils.put(token,number,EXPIRE_TIME_S);
            return token;
        } catch (UnsupportedEncodingException e) {
            return null;
        }
    }

    /**
     * 校验 token 是否正确
     */
    public static boolean verify(String token, String number) {
        try {
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            //在token中附带了username信息
            JWTVerifier verifier = JWT.require(algorithm)
                    .withClaim("number", number)
                    .build();
            //验证 token
            verifier.verify(token);
            return true;
        } catch (Exception exception) {
            return false;
        }
    }

    /**
     * 获得token中的信息，无需secret解密也能获得
     */
    public static String getNumber(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim("number").asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }
    /**
     * 获得token中的信息，无需secret解密也能获得
     */
    public static String getCacheNumber(String token) {
        try {
            return (String) HutoolCacheUtils.get(token);
        } catch (JWTDecodeException e) {
            return null;
        }
    }
    /**
     * 获得token中的信息，无需secret解密也能获得
     */
    public static String getUserId(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim("userid").asString();
        } catch (JWTDecodeException e) {
            return null;
        }
    }



}